package com.cds.repository;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;

import com.cds.model.Article;
import com.cds.model.Body;
import com.cds.model.Comments;
import com.cds.model.Title;
import com.cds.util.UserArticle;
import com.mongodb.BasicDBObject;
import com.mongodb.CommandResult;

public class ArticleRepository {

	@Autowired
	private MongoTemplate mongoTemplate;
	ApplicationContext ctx ;
	
	public ArticleRepository(){
		ctx = ApplicationSingleton.getInstance().getContext();
		mongoTemplate = (MongoTemplate) ctx.getBean("mongoTemplate");
	}

	
	/***
	 * Method to create a new article 
	 * @param article
	 */
	public  void createArticle(Article article) {
		mongoTemplate.insert(article);
	}
	
	/**
	 * Method to find an article by document id, This method will check for user permissions, 
	 * based on user permission different article attributes will be visible to the user
	 * if the user if anonymous user, then only public attributes are presented to the user. 
	 * if the user has right permission then both user content and public content are visible to the user 
	 * @param docid
	 * @param user
	 * @return return Article
	 */
	public Article findArticleByID(String docid,String user){ 
		Query query = new Query();
		UserArticle uArticle = new UserArticle(mongoTemplate);
		List<String> includeKeys = null;
		/** get the user permission to view this article**/
		String permission = uArticle.getUserPermissions(user);
		System.out.println("findByArticleID() Permission : "+permission);
		/**verify if user has any permissions, if not he/she is anonymous user*/
		if(permission == null) 
		{
			/** if user is anonymous then determine what attribute does this anonymous user should see ( for now only public content)**/
			includeKeys = uArticle.getIncludedKeysForAnonymousUser(docid);
			if(includeKeys !=null && includeKeys.size() > 0){
				for (String include : includeKeys) {
					System.out.println("Included Key for Anonymous User : "+include);
					query.fields().include(include);  // show only these keys for this USER
				}
				Criteria criteria = Criteria.where("docid").is(docid);
				query.addCriteria(criteria);
	
			}else{ 
				 /** there is nothing to be show, as none of the fields are public then return empty article.**/
				return new Article();
			}
		}
		else
		{  
			/** if this is a valid user then get all the keys for user which are valid including public content**/
			includeKeys = uArticle.getIncludedKeys(permission,docid);
			if(includeKeys !=null && includeKeys.size() > 0)
			{
				for (String include : includeKeys) {
					System.out.println("Included Key : "+include);
					query.fields().include(include);  // show only these keys for this USER
				}
			}else{
				/** if there no include key's , it means user is not privileged to see any thing as there is no public content associated with this article**/
				return new Article();
			}
			/***Construct Criteria with specific attributes for this document id which user can view*/
			Criteria criteria = Criteria.where("docid").is(docid);
			query.addCriteria(criteria);
		}
	 
		Article article = mongoTemplate.findOne(query,Article.class);
		// process this article to not show the permission tag.
		//article = removePermissionTag(article);
		return article;

	}
	
	public Article removePermissionTag(Article article){
		
		Article a = new Article();
		Body b = new Body();
		Title t = new Title();
		Comments c  = new Comments();
		
		a.setDocid(article.getDocid());
		a.setId(article.getId());
		
		c.setComments(article.getComments().getComments());
		b.setContent(article.getBody().getContent());
		t.setName(article.getTitle().getName());
		
		a.setBody(b);
		a.setTitle(t);
		a.setComments(c);
		return a;
	}
	
	
	/***
	 * method to find all the articles for a given locale, currently no security is added here.
	 * @param locale
	 * @return return list of articles
	 */
	public List<Article> findArticleByLocale(String locale){
		Query query = new Query();
		query.sort();
 		Criteria criteria = Criteria.where("locale").is(locale);
		query.addCriteria(criteria);
		List<Article> articles = mongoTemplate.find(query,Article.class);
		return articles;

	}
	/**
	 * method to find all the articles for the specific user, if no user is specified or if he/she is anonymous user then only show public content
	 * @param user
	 * @return returns list of articles
	 */
	public List<Article> findAll(String user){
		List<Article> articles= new ArrayList<Article>();
		UserArticle uArticle = new UserArticle(mongoTemplate);
		/** get the user permission to view articles**/
		String permission = uArticle.getUserPermissions(user);
		System.out.println("findAll() Permission : "+permission);
		/** if the user is anonymous user then only show public articles ***/
		if(permission == null){
			Query query = new Query();
			query.sort();
			Criteria criteria = Criteria.where("permissions").is("public");
			query.addCriteria(criteria);
 			articles = mongoTemplate.find(query,Article.class);
 			
		}else{
			/**if he/she is a valid user then show user all the content which are tagged with user's permissions as well as public content**/
 			System.out.println("findAll() Permission : "+permission);
			if(permission != null ){
				Query query = new Query();
				query.sort();
		 		Criteria criteria = new Criteria().orOperator(Criteria.where("permissions").is(permission),  
	                    Criteria.where("permissions").is("public"));
				query.addCriteria(criteria);
				articles = mongoTemplate.find(query,Article.class);
			} 
		}
		return articles;
	}
	/***
	 * method for full-text-search
	 * @param search
	 * @return return the articles which has the search term
	 */
	public CommandResult searchArticles(String search){
		// make sure you set index on the attribute you want to search
		mongoTemplate.getCollection("article").ensureIndex(new BasicDBObject("body", "text"));
		
		BasicDBObject textSearch = new BasicDBObject();
		textSearch.put("text", "article");
		textSearch.put("search", search);
		
	return mongoTemplate.executeCommand(textSearch);
 	}
	
	 
}
